#include "Figure.h"

#define _USE_MATH_DEFINES
#include <cmath>

#include "Point.h"
#include "Line.h"

namespace mas {

Figure::~Figure ()
{
}

void Figure::RotateAround (double angle, const Point& p)
{
  Translate (-p.X(), -p.Y());
  Rotate (angle);
  Translate (p.X(), p.Y());
}

void Figure::RotateByDegrees (double degrees)
{
  Rotate (degrees * M_PI / 180);
}

void Figure::RotateAroundByDegrees (double degrees, const Point& p)
{
    RotateAround (degrees * M_PI / 180, p);
}

void Figure::ScaleByPoint (const Point& p)
{
  Scale (p.X (), p.Y ());
}

void Figure::ScaleUniformly(double f)
{
  Scale (f, f);
}

void Figure::TranslateByPoint (const Point& p)
{
  Translate (p.X (), p.Y ());
}

void Figure::TranslateByPointReverse (const Point& p)
{
  Translate (-p.X (), -p.Y ());
}

void Figure::Mirror (const Line& z)
{
  Point p(z.Intersection(z.Perpendicular(Point())));
  Translate(-p.X(), -p.Y());
  Rotate(-z.Angle());
  Scale(1.0, -1.0);
  Rotate(z.Angle());
  Translate(p.X(), p.Y());
}

void Figure::WriteLine (std::ostream& output) const
{
  Write(output);
  output << std::endl;
}

std::ostream& operator << (std::ostream& output, const Figure& f)
{
  f.Write(output);
  return output;
}

bool Figure::IsRotatable () const
{
  return true;
}

bool Figure::IsDeformable () const
{
  return true;
}

}
