#ifndef _H_Line
#define _H_Line

#include "Figure.h"
#include "Point.h"
#include "Canvas.h"

namespace mas {

class Line: public Figure {
private:
  double a_;
  double b_;
  double c_;
// invariant a_ != 0.0 || b_ != 0.0;
public:
  Line ();
  Line (const Point& p1, const Point& p2);  // pre p1 != p2;
  Line (const Point& p, double angle);
  Line (const Line& other);
  Line (double a, double b, double c);  // pre a != 0.0 || b != 0.0;
  virtual ~Line ();

  virtual Line* Clone () const;

  virtual bool operator == (const Line &other) const;
  virtual bool operator != (const Line &other) const;

  virtual double Angle () const;
  virtual double X0 () const; // pre !Horizontal();
  virtual double Y0 () const; // pre !Vertical();

  virtual double Value (const Point& p) const;

  virtual bool Vertical () const;
  virtual bool Horizontal () const;

  virtual bool Has (const Point& p) const;
  virtual bool Between (const Point& p1, const Point& p2) const;
  virtual bool SameSide (const Point& p1, const Point& p2) const;

  virtual bool IsParallel (const Line& other) const;
  virtual Point Intersection (const Line& other) const; // pre !IsParallel(other);

  virtual Line Parallel (const Point& p) const;
  virtual Line Perpendicular (const Point& p) const;
  virtual double DistanceTo (const Point& p) const;
  
  virtual void Scale (double fx, double fy);  // pre fx != 0.0 || fy != 0.0;
  virtual void Rotate (double angle);
  virtual void Translate (double dx, double dy);

  virtual void Draw (Canvas& x) const;
  virtual void Write (std::ostream& output = std::cout) const;

};

}

#endif
