#include <iostream>

#include "Figure.h"
#include "Point.h"
#include "Particle.h"

namespace mas {

Particle::Particle ():
  Point (),
  mass_ (1.0)
{
}

Particle::Particle (double x, double y, double mass):
  Point (x, y),
  mass_ (mass)
{
}

Particle::Particle (const Particle& other):
  Point (other),
  mass_ (other.mass_)
{
}

Particle::Particle (const Point& other):
  Point (other),
  mass_ (1.0)
{
}

Particle::~Particle ()
{
}

double Particle::Energy () const
{
  return mass_ * speedOfLight * speedOfLight;
}

void Particle::Energize (double energy)
{
  mass_ += energy / (speedOfLight * speedOfLight);
}

double Particle::Mass () const
{
  return mass_;
}

double Particle::Force (const Particle& other) const
{
  double d = DistanceTo (other);
  return gravitational * mass_ * other.mass_ / (d * d);
}

void Particle::Write (std::ostream& output) const
{
  Point::Write(output);
  output << " " << mass_;
}

const double Particle::speedOfLight = 299792458;  //meter / second
const double Particle::gravitational = 6.6742e-11;  // Newton*meter^2 / second^2

}
