#include "Point.h"

//#define _USE_MATH_DEFINES
#include <cmath>

namespace mas {

Point::Point ():
  x_ (0),
  y_ (0)
{
}

Point::Point (double x, double y):
  x_ (x),
  y_ (y)
{
}

Point::Point (const Point& other):
  x_ (other.x_),
  y_ (other.y_)
{
}

Point::~Point ()
{
}

Point* Point::Clone() const
{
  return new Point (*this);
}

bool Point::Equal (const Point& other) const
{
  return x_ == other.x_ && y_ == other.y_;
}

bool Point::operator == (const Point& other) const
{
  return x_ == other.x_ && y_ == other.y_;
}

bool Point::operator != (const Point& other) const
{
  return ! operator == (other);
}

bool Point::operator <= (const Point& other) const
{
  //return x_ < other.x_ || x_ == other.x_ && y_ <= other.y_;
  return ! (other.x_ <= x_) || x_ == other.x_ && y_ <= other.y_;
}

bool Point::operator < (const Point& other) const
{
  //return ! other.operator <= (*this);
  return ! (other.x_ < x_) && (!(x_ == other.x_) || y_ < other.y_);
}

bool Point::operator >= (const Point& other) const
{
  return ! operator < (other);
}

bool Point::operator > (const Point& other) const
{
  return ! operator <= (other);
}

double Point::X () const
{
  return x_;
}

double Point::Y () const
{
  return y_;
}

void Point::SetX (double x)
{
  x_ = x;
}

void Point::SetY (double y)
{
  y_ = y;
}

void Point::Set (double x, double y)
{
  x_ = x;
  y_ = y;
}

void Point::Translate (double dx, double dy)
{
  x_ += dx;
  y_ += dy;
}

void Point::Scale (double fx, double fy)
{
  x_ *= fx;
  y_ *= fy;
}

void Point::Rotate(double angle)
{
  double x0 = x_;
  double y0 = y_;
  x_ = x0 * ::cos (angle) - y0 * ::sin (angle);
  y_ = x0 * ::sin (angle) + y0 * ::cos (angle);
}

double Point::DistanceTo (const Point& other) const
{
  return ::sqrt (::pow (x_ - other.x_, 2) + ::pow (y_ - other.y_, 2));
}

double Point::Angle () const
{
  return ::atan2 (y_, x_);
}

double Point::Modulus () const
{
  return ::sqrt (x_*x_ + y_*y_);
}

void Point::Draw (Canvas& x) const
{
  x.DrawPoint (*this);
}

void Point::Write (std::ostream& output) const
{
  output << x_ << " " << y_;
}

void Point::Read (std::istream& input)
{
  input >> x_ >> y_;
}

}
