#ifndef _H_Utilities
#define _H_Utilities

#include <iostream>
#include <fstream>
#include <iomanip>
#include <string>
#include <vector>
#include <list>
#include <algorithm>
#include <numeric>
#define _USE_MATH_DEFINES
#include <cmath>
#include <limits>
#include <ctime>
#include <cstdarg>

namespace mas {

template <class T>
void Sort(std::vector<T>& v)
{
  std::sort(v.begin(), v.end());
}

template <class T>
void Unique(std::vector<T>& v)
{
  v.erase(std::unique(v.begin(), v.end()), v.end());
}

template <class T>
void Reverse(std::vector<T>& v)
{
  for (int i = 0, j = static_cast<int>(v.size()) - 1; i < j; i++, j--)
    std::swap(v[i], v[j]);
}

template <class T>
void Shuffle (std::vector<T>& v)
{
  int n = static_cast<int> (v.size ());
  for (int i = 0; i < 2*n; i++)
    std::swap (v[rand () % n], v[rand () % n]);
}

//template <class T>
//bool Find(const std::vector<T>& v, T x)
//{
//  return std::find(v.begin(), v.end(), x) != v.end();
//}
//
//template <class T>
//bool BinarySearch(const std::vector<T>& v, T x)
//{
//  return std::binary_search(v.begin(), v.end(), x);
//}
//
//template <class T>
//bool Find(const std::list<T>& v, T x)
//{
//  return std::find(v.begin(), v.end(), x) != v.end();
//}

template <class T>
T Min(T x, T y)
{
  return x <= y ? x : y;
}

template <class T>
T Max(T x, T y)
{
  return x <= y ? y : x;
}

template <class T>
T Min(T x, T y, T z)
{
  return Min(Min(x, y), z);
}

template <class T>
T Max(T x, T y, T z)
{
  return Max(Max(x, y), z);
}

template <class T>
void Sort(T& x, T& y)
{
  if (!(x <= y))
    std::swap(x, y);
}

template <class T>
void Sort(T& x, T& y, T& z)
{
  Sort(y, z);
  Sort(x, y);
  Sort(y, z);
}

template <class T>
bool Sorted(T& x, T& y, T& z)
{
  return x <= y && y <= z;
}



// Returns the order of the first argument after the first which is equal to the first
template <class T>
int Match(T x, ...)
{
  va_list p;
  va_start(p, x);
  int result = 0;
  while (static_cast<T>(va_arg(p, T)) != x)
    result++;
  va_end(p);
  return result;
}

template <class T>
T Square(T x)
{
  return x * x;
}

template <class T>
T Cube(T x)
{
  return x * x * x;
}

template <class T>
void Write (const std::vector<T>& v, const std::string& separator = " ", std::ostream& output = std::cout)
{
  for (typename std::vector<T>::const_iterator i = v.begin(); i != v.end(); i++)
    output << (i != v.begin() ? separator : "") << *i;
}

template <class T>
void Write(const std::vector<T>& v, int width, std::ostream& output = std::cout)
{
  for (typename std::vector<T>::const_iterator i = v.begin(); i != v.end(); i++)
    output << std::setw(width) << *i;
}

template <class T>
void WriteIndirect(const std::vector<T>& v, const std::list<int>& x, const std::string& separator = " ", std::ostream& output = std::cout)
{
  for (std::list<int>::const_iterator i = x.begin(); i != x.end(); i++)
    output << (i != x.begin() ? separator : "") << v[*i];
}

template <class T>
void WriteIndirect(const std::vector<T>& v, const std::list<int>& x, int width, std::ostream& output = std::cout)
{
  for (std::list<int>::const_iterator i = x.begin(); i != x.end(); i++)
    output << std::setw(width) << v[*i];
}

template <class T>
void WriteBackwards(const std::vector<T>& v, const std::string& separator = " ", std::ostream& output = std::cout)
{
  for (typename std::vector<T>::const_reverse_iterator i = v.rbegin(); i != v.rend(); i++)
    output << (i != v.rbegin() ? separator : "") << *i;
}

template <class T>
void WriteBackwards(const std::vector<T>& v, int width, std::ostream& output = std::cout)
{
  for (typename std::vector<T>::const_reverse_iterator i = v.rbegin(); i != v.rend(); i++)
    output << std::setw(width) << *i;
}

template <class T>
void WriteLine (const std::vector<T>& v, const std::string& separator = " ", std::ostream& output = std::cout)
{
  Write(v, separator, output);
  output << std::endl;
}

template <class T>
void WriteLine (const std::vector<T>& v, int width, std::ostream& output = std::cout)
{
  Write(v, width, output);
  output << std::endl;
}

template <class T>
void WriteBackwardsLine(const std::vector<T>& v, const std::string& separator = " ", std::ostream& output = std::cout)
{
  WriteBackwards(v, separator, output);
  output << std::endl;
}

template <class T>
void WriteBackwardsLine(const std::vector<T>& v, int width, std::ostream& output = std::cout)
{
  WriteBackwards(v, width, output);
  output << std::endl;
}

template <class T>
void WriteIndirectLine(const std::vector<T>& v, const std::list<int>& x, const std::string& separator = " ", std::ostream& output = std::cout)
{
  WriteIndirect(v, x, separator, output);
  output << std::endl;
}

template <class T>
void WriteIndirectLine(const std::vector<T>& v, const std::list<int>& x, int width, std::ostream& output = std::cout)
{
  WriteIndirect(v, x, width, output);
  output << std::endl;
}


template <class T, class U>
void Write2 (const std::vector<std::pair<T, U> >& v, const std::string& separator1 = " ", const std::string& separator2 = " ", std::ostream& output = std::cout)
{
  for (typename std::vector<std::pair<T, U> >::const_iterator i = v.begin(); i != v.end(); i++)
    output << (i != v.begin() ? separator1 : "") << i -> first << separator2 << i -> second;
}

template <class T, class U>
void Write2Backwards(const std::vector<std::pair<T, U> >& v, const std::string& separator1 = " ", const std::string& separator2 = " ", std::ostream& output = std::cout)
{
  for (typename std::vector<std::pair<T, U> >::const_reverse_iterator i = v.rbegin(); i != v.rend(); i++)
    output << (i != v.rbegin() ? separator1 : "") << i -> first << separator2 << i -> second;
}

template <class T, class U>
void Write2Line (const std::vector<std::pair<T, U> >& v, const std::string& separator1 = " ", const std::string& separator2 = " ", std::ostream& output = std::cout)
{
  Write2(v, separator1, separator2, output);
  output << std::endl;
}

template <class T, class U>
void Write2BackwardsLine(const std::vector<std::pair<T, U> >& v, const std::string& separator1 = " ", const std::string& separator2 = " ", std::ostream& output = std::cout)
{
  Write2Backwards(v, separator1, separator2, output);
  output << std::endl;
}

template <class T>
void Write(const std::list<T>& v, const std::string& separator = " ", std::ostream& output = std::cout)
{
  for (typename std::list<T>::const_iterator i = v.begin(); i != v.end(); i++)
    output << (i != v.begin() ? separator : "") << *i;
}

template <class T>
void Write(const std::list<T>& v, int width, std::ostream& output = std::cout)
{
  for (typename std::list<T>::const_iterator i = v.begin(); i != v.end(); i++)
    output << std::setw(width) << *i;
}

template <class T>
void WriteLine(const std::list<T>& v, const std::string& separator = " ", std::ostream& output = std::cout)
{
  Write(v, separator, output);
  output << std::endl;
}

template <class T>
void WriteLine(const std::list<T>& v, int width, std::ostream& output = std::cout)
{
  Write(v, width, output);
  output << std::endl;
}

template <class T, class U>
void Write2(const std::list<std::pair<T, U> >& v, const std::string& separator1 = " ", const std::string& separator2 = " ", std::ostream& output = std::cout)
{
  for (typename std::list<std::pair<T, U> >::const_iterator i = v.begin(); i != v.end(); i++)
    output << (i != v.begin() ? separator1 : "") << i -> first << separator2 << i -> second;
}

template <class T, class U>
void Write2Backwards(const std::list<std::pair<T, U> >& v, const std::string& separator1 = " ", const std::string& separator2 = " ", std::ostream& output = std::cout)
{
  for (typename std::list<std::pair<T, U> >::const_reverse_iterator i = v.rbegin(); i != v.rend(); i++)
    output << (i != v.rbegin() ? separator1 : "") << i -> first << separator2 << i -> second;
}

template <class T, class U>
void Write2Line(const std::list<std::pair<T, U> >& v, const std::string& separator1 = " ", const std::string& separator2 = " ", std::ostream& output = std::cout)
{
  Write2(v, separator1, separator2, output);
  output << std::endl;
}

template <class T, class U>
void Write2BackwardsLine(const std::list<std::pair<T, U> >& v, const std::string& separator1 = " ", const std::string& separator2 = " ", std::ostream& output = std::cout)
{
  Write2Backwards(v, separator1, separator2, output);
  output << std::endl;
}

template <class T>
void WriteV(const std::vector<std::vector<T> >& v, const std::string& separator1 = " ", const std::string& separator2 = " ", std::ostream& output = std::cout)
{
  for (typename std::vector<std::vector<T> >::const_iterator i = v.begin(); i != v.end(); i++)
  {
    if (i != v.begin ())
      output << separator1;
    Write(*i, separator2, output);
  }
}

template <class T>
void WriteV(const std::vector<std::vector<T> >& v, const std::string& separator1, int width, std::ostream& output = std::cout)
{
  for (typename std::vector<std::vector<T> >::const_iterator i = v.begin(); i != v.end(); i++)
  {
    if (i != v.begin ())
      output << separator1;
    Write(*i, width, output);
  }
}

template <class T>
void WriteVLine(const std::vector<std::vector<T> >& v, const std::string& separator1 = " ", const std::string& separator2 = " ", std::ostream& output = std::cout)
{
  WriteV(v, separator1, separator2, output);
  output << std::endl;
}

template <class T>
void WriteVLine(const std::vector<std::vector<T> >& v, const std::string& separator1 , int width, std::ostream& output = std::cout)
{
  WriteV(v, separator1, width, output);
  output << std::endl;
}

template <class T>
void WriteL(const std::list<std::list<T> >& v, const std::string& separator1 = " ", const std::string& separator2 = " ", std::ostream& output = std::cout)
{
  for (typename std::list<std::list<T> >::const_iterator i = v.begin(); i != v.end(); i++)
  {
    if (i != v.begin ())
      output << separator1;
    Write(*i, separator2, output);
  }
}

template <class T>
void WriteL(const std::list<std::list<T> >& v, const std::string& separator1, int width, std::ostream& output = std::cout)
{
  for (typename std::list<std::list<T> >::const_iterator i = v.begin(); i != v.end(); i++)
  {
    if (i != v.begin ())
      output << separator1;
    Write(*i, width, output);
  }
}

template <class T>
void WriteLLine(const std::list<std::list<T> >& v, const std::string& separator1 = " ", const std::string& separator2 = " ", std::ostream& output = std::cout)
{
  WriteL(v, separator1, separator2, output);
  output << std::endl;
}

template <class T>
void WriteLLine(const std::list<std::list<T> >& v, const std::string& separator1, int width, std::ostream& output = std::cout)
{
  WriteL(v, separator1, width, output);
  output << std::endl;
}

template <class T>
void Append (std::vector<T>& v, const std::vector<T>& u)
{
  for (typename std::vector<T>::const_iterator i = u.begin(); i != u.end(); i++)
    v.push_back (*i);
}

template <class T>
void Append (std::vector<T>& v, typename std::vector<T>::iterator first, typename std::vector<T>::iterator last)
{
  for (typename std::vector<T>::const_iterator i = first; i != last; i++)
    v.push_back (*i);
}

template <class T>
std::ostream& operator << (std::ostream& output, const std::vector<T>& v)
{
  Write(v, " ", output);
  return output;
}

template <class T>
std::ostream& operator << (std::ostream& output, const std::list<T>& v)
{
  Write(v, " ", output);
  return output;
}

template <class T, class U>
bool OrderedLessFirst(const std::pair<T, U>& x, const std::pair<T, U>& y)
{
  return x.first <= y.first;
}

template <class T, class U>
bool OrderedGreaterFirst(const std::pair<T, U>& x, const std::pair<T, U>& y)
{
  return y.first <= x.first;
}

template <class T, class U>
bool OrderedLessSecond(const std::pair<T, U>& x, const std::pair<T, U>& y)
{
  return x.second <= y.second;
}

template <class T, class U>
bool OrderedGreaterSecond(const std::pair<T, U>& x, const std::pair<T, U>& y)
{
  return y.second <= x.second;
}

template <class T, class U>
bool OrderedLessFirstLessSecond(const std::pair<T, U>& x, const std::pair<T, U>& y)
{
  //return x.first < y.first || x.first == y.first && x.second <= y.second;
  return !(y.first <= x.first) || x.first == y.first && x.second <= y.second;
}

template <class T, class U>
bool OrderedLessFirstGreaterSecond(const std::pair<T, U>& x, const std::pair<T, U>& y)
{
  return !(y.first <= x.first) || x.first == y.first && y.second <= x.second;
}

template <class T, class U>
bool OrderedGreaterFirstLessSecond(const std::pair<T, U>& x, const std::pair<T, U>& y)
{
  return !(x.first <= y.first) || x.first == y.first && x.second <= y.second;
}

template <class T, class U>
bool OrderedGreaterFirstGreaterSecond(const std::pair<T, U>& x, const std::pair<T, U>& y)
{
  return !(x.first <= y.first) || x.first == y.first && y.second <= x.second;
}


template <class T, class U>
bool OrderedLessSecondLessFirst(const std::pair<T, U>& x, const std::pair<T, U>& y)
{
  return !(y.second <= x.second) || x.second == y.second && x.first <= y.first;
}

template <class T, class U>
bool OrderedLessSecondGreaterFirst(const std::pair<T, U>& x, const std::pair<T, U>& y)
{
  return !(y.second <= x.second) || x.second == y.second && y.first <= x.first;
}

template <class T, class U>
bool OrderedGreaterSecondLessFirst(const std::pair<T, U>& x, const std::pair<T, U>& y)
{
  return !(x.second <= y.second) || x.second == y.second && x.first <= y.first;
}

template <class T, class U>
bool OrderedGreaterSecondGreaterFirst(const std::pair<T, U>& x, const std::pair<T, U>& y)
{
  return !(x.second <= y.second) || x.second == y.second && y.first <= x.first;
}


template <class T, class U>
bool LessFirst(const std::pair<T, U>& x, const std::pair<T, U>& y)
{
  return x.first < y.first;
}

template <class T, class U>
bool GreaterFirst(const std::pair<T, U>& x, const std::pair<T, U>& y)
{
  return y.first < x.first;
}

template <class T, class U>
bool LessSecond(const std::pair<T, U>& x, const std::pair<T, U>& y)
{
  return x.second < y.second;
}

template <class T, class U>
bool GreaterSecond(const std::pair<T, U>& x, const std::pair<T, U>& y)
{
  return y.second < x.second;
}

template <class T, class U>
bool LessFirstLessSecond(const std::pair<T, U>& x, const std::pair<T, U>& y)
{
  return !(y.first < x.first) && (!(x.first == y.first) || x.second < y.second);
}

template <class T, class U>
bool LessFirstGreaterSecond(const std::pair<T, U>& x, const std::pair<T, U>& y)
{
  return !(y.first < x.first) && (!(x.first == y.first) || y.second < x.second);
}

template <class T, class U>
bool GreaterFirstLessSecond(const std::pair<T, U>& x, const std::pair<T, U>& y)
{
  return !(x.first < y.first) && (!(x.first == y.first) || x.second < y.second);
}

template <class T, class U>
bool GreaterFirstGreaterSecond(const std::pair<T, U>& x, const std::pair<T, U>& y)
{
  return !(x.first < y.first) && (!(x.first == y.first) || y.second < x.second);
}


template <class T, class U>
bool LessSecondLessFirst(const std::pair<T, U>& x, const std::pair<T, U>& y)
{
  return !(y.second < x.second) && (!(x.second == y.second) || x.first < y.first);
}

template <class T, class U>
bool LessSecondGreaterFirst(const std::pair<T, U>& x, const std::pair<T, U>& y)
{
  return !(y.second < x.second) && (!(x.second == y.second) || y.first < x.first);
}

template <class T, class U>
bool GreaterSecondLessFirst(const std::pair<T, U>& x, const std::pair<T, U>& y)
{
  return !(x.second < y.second) && (!(x.second == y.second) || x.first < y.first);
}

template <class T, class U>
bool GreaterSecondGreaterFirst(const std::pair<T, U>& x, const std::pair<T, U>& y)
{
  return !(x.second < y.second) && (!(x.second == y.second) || y.first < x.first);
}

template <class T>
void Bubblesort (std::vector<T>& v)
{
  int n = static_cast<int> (v.size ());
  for (int i = 1; i < n; i++)
    for (int j = n - 1; j >= i; j--)
      if (v[j-1] > v[j])
        std::swap (v[j-1], v[j]);
}

template <class T>
bool BinarySearch (const std::vector <T>& v, const T& x)
{
  bool result = false;
  int i = 0;
  int j = static_cast<int> (v.size ());
  while (!result && i <= j)
  {
    int m = (i+j) / 2;
    if (v[m] < x)
      i = m + 1;
    else if (x < v[m])
      j = m - 1;
    else
      result = true;
  }
  return result;
}

template <class T>
typename std::vector<T>::iterator Find (std::vector<T>& v, const T& x)
{
  typename std::vector<T>::iterator result = v.begin ();
  while (result != v.end () && !(*result == x))
    result++;
  return result;
}

}

#endif
